﻿<a id="markdown-известные-проблемы-в-vanessaadd" name="известные-проблемы-в-vanessaadd"></a>
# Известные проблемы в Vanessa.ADD

<!-- TOC -->

- [Известные проблемы в Vanessa.ADD](#известные-проблемы-в-vanessaadd)
    - [TDD/xdd - при запуске через xxdTestRunner](#tddxdd---при-запуске-через-xxdtestrunner)
        - [Выполнение тестов останавливается](#выполнение-тестов-останавливается)
        - [Ошибки работы с транзакциями](#ошибки-работы-с-транзакциями)
        - ["Странные ошибки"](#странные-ошибки)
        - [Запуск под пользователем с ограниченными правами.](#запуск-под-пользователем-с-ограниченными-правами)
    - [BDD - при запуске через bddRunner](#bdd---при-запуске-через-bddrunner)
        - [Основной FAQ для BDD](#основной-faq-для-bdd)
        - [При попытке подключить TestClient подключение не происходит.](#при-попытке-подключить-testclient-подключение-не-происходит)
            - [Вариант 1 (TMP Metadata)](#вариант-1-tmp-metadata)
            - [Вариант 2 (Windows Firewall)](#вариант-2-windows-firewall)
        - [Проблемы с выполнением библиотечных шагов](#проблемы-с-выполнением-библиотечных-шагов)

<!-- /TOC -->

<a id="markdown-tddxdd---при-запуске-через-xxdtestrunner" name="tddxdd---при-запуске-через-xxdtestrunner"></a>
## TDD/xdd - при запуске через xxdTestRunner

<a id="markdown-выполнение-тестов-останавливается" name="выполнение-тестов-останавливается"></a>
### Выполнение тестов останавливается

1. Проблема: При загрузке тестов в тонком клиенте управляемого приложения 8.2.Х.Х могут возникать исключения и загрузка тестов может останавливаться.
  1. Это платформенная ошибка 1С в управляемых формах. Проблема детально описана в задачах [#86](https://github.com/xDrivenDevelopment/xUnitFor1C/issues/86) или [#154](https://github.com/xDrivenDevelopment/xUnitFor1C/issues/154) или [#256](https://github.com/xDrivenDevelopment/xUnitFor1C/issues/256).
  1. Решение 1: Исправьте ошибку, приводящую к остановке загрузки. Перезагрузите тесты
  1. Решение 2: Используйте платформу 8.3.4 и выше, там эта платформенная ошибка исправлена.

1. Аналогичная ошибка наблюдается в толстом клиенте обычное приложение 8.2.Х.Х и в 8.3.5.Х при открытии форм, в которых есть выброс исключения внутри кода открытия формы. Например, [Описание ошибки](http://xdd.silverbulleters.org/t/kak-pojmat-padenie-v-pri-otkrytii-formy/111/6)

<a id="markdown-ошибки-работы-с-транзакциями" name="ошибки-работы-с-транзакциями"></a>
### Ошибки работы с транзакциями

При ошибках работы с транзакциями, например, при вызове `ОтменитьТранзакцию` нужно использовать следующую конструкцию:

```bsl
Если ТранзакцияАктивна() Тогда
  ОтменитьТранзакцию();
КонецЕсли;
```

<a id="markdown-странные-ошибки" name="странные-ошибки"></a>
### "Странные ошибки"

1. При попытке **загрузки тестов в своих ИБ** могут появляться "странные" сообщения/предупреждения/ошибки (например, [см. баг #458](https://github.com/xDrivenDevelopment/xUnitFor1C/issues/458)).
    1. Проблема может быть в том, что глобальные обработчики оповещения, подключенные через `ПодключитьОбработчикОповещения`, не умеют пропускать чужие оповещения. Vanessa.ADD использует оповещения в своей форме для загрузки/выполнения тестов. Все оповещения Vanessa.ADD начинаются на префикс `xUnitFor1C.`

        1. Рекомендуется глобальный поиск по ПодключитьОбработчикОповещения и доработку кода всех найденных обработчиков оповещений.

    1. В случае иных ошибок рекомендуется запустить 1С:Предприятие в режиме отладки, попытаться загрузить тесты и поймать место возникновения ошибки.
        1. Если код ошибки находится внутри кода Vanessa.ADD, [создавайте баг для Vanessa.ADD](https://github.com/vanessa-opensource/add/issues/new)
        1. Если код ошибки находится внутри вашей конфигурации, исправляйте проблему у себя.


1. Если в пустой конфигурации в упр.приложении запустить xddTestRunner, при открытии внешнего инструмента "Показать ГУИД" и попытке выбора ссылки 1С:Предприятие падает. [Описание бага #304](https://github.com/xDrivenDevelopment/xUnitFor1C/issues/304)

<a id="markdown-запуск-под-пользователем-с-ограниченными-правами" name="запуск-под-пользователем-с-ограниченными-правами"></a>
### Запуск под пользователем с ограниченными правами.

Если не встраивать обработки в конфигурацию и есть желание запустить тесты(например: дымовые) под пользователем с ограниченными правами, возникает ошибка "Установлен безопасный режим. Выполнение операции запрещено".

Для возможности запуска необходимы минимальный набор прав
    + "Администрирование",
    + "Интерактивное открытие внешних обработок".

<a id="markdown-bdd---при-запуске-через-bddrunner" name="bdd---при-запуске-через-bddrunner"></a>
## BDD - при запуске через bddRunner

<a id="markdown-основной-faq-для-bdd" name="основной-faq-для-bdd"></a>
### Основной FAQ для BDD

+ часто задаваемые вопросы [FAQ.md](/F.A.Q.MD)

<a id="markdown-при-попытке-подключить-testclient-подключение-не-происходит" name="при-попытке-подключить-testclient-подключение-не-происходит"></a>
### При попытке подключить TestClient подключение не происходит.

<a id="markdown-вариант-1-tmp-metadata" name="вариант-1-tmp-metadata"></a>
#### Вариант 1 (TMP Metadata)

Это проблема платформы 1С и выражается в плавающей ошибке "Ошибка сетевого взаимодействия" - наличествует в версиях до 8.3.7.1917 включительно

* Лечится закрытием всех сеансов 1С (в том числе конфигуратора) и запуском снова.
* Ну и от перезагрузки ещё никто не уходил и не забывайте чистить %TMP% каталог регулярно на машине разработчика

<a id="markdown-вариант-2-windows-firewall" name="вариант-2-windows-firewall"></a>
#### Вариант 2 (Windows Firewall)

Это проблема странной установки 1С предприятия и первого запуска:

* когда вы только установили очередную версию платформы, при первом старте 1С Windows вас спрашивает резрешить ли сетевое взаимодействие  для процесс 1cv8.exe - некоторые товарищи нажимают Отмена в этом случае
* тогда у вас в Windows Защитнике сетевое взаимодействие по отладочным портам будет заблокировано - так как комуникация идет между двумя процессами 1cv8.exe и 1cv8c.exe - то есть между тонким клиентом и толстым клиентов
* Windows такое взаимодействие считает межпроцессным сетевым соединением и накрывает правилами брэндмауэра

лечить это достаточно просто и одновременно сложно

* зайти в Windows и создать разрешающее правило для процессов
* какие порты указывать - разберитесь сами ;-), чтобы в следующий раз смотрели на возникающие окна при установке ;-)

<a id="markdown-проблемы-с-выполнением-библиотечных-шагов" name="проблемы-с-выполнением-библиотечных-шагов"></a>
### Проблемы с выполнением библиотечных шагов

1. Вы используете старый шаг **В открытой форме из выпадающего списка я выбираю "что-то"**, и он не работает

2. Вариант решения

Надо заменить текст шага вручную или накликать его заново.

Варианты замены:

* В открытой форме из выпадающего списка "мой список" я выбираю "что-то"
* В открытой форме из выпадающего списка с заголовком "мой список" я выбираю "что-то"
* В открытой форме из выпадающего списка с именем "МойСписок" я выбираю "что-то"

В первых двух вариантах поиск будет идти по заголовку элемента формы, в третьем - по внутреннему имени, которое задано в конфигураторе.
