# Генерация данных

<!-- TOC -->

- [Генерация данных](#генерация-данных)
    - [Режимы генерации данных при создании объектов и поиске реквизитов объектов](#режимы-генерации-данных-при-создании-объектов-и-поиске-реквизитов-объектов)
        - [Режимы создания](#режимы-создания)
        - [Режимы поиска](#режимы-поиска)
    - [Генерация макета на базе реальных данных](#генерация-макета-на-базе-реальных-данных)
        - [Пример кода создания данных из макета](#пример-кода-создания-данных-из-макета)
    - [Создание макета данных на основании результатов запросов (например, через ирПортативные)](#создание-макета-данных-на-основании-результатов-запросов-например-через-ирпортативные)
        - [Пример вызова](#пример-вызова)
    - [Пример запросов  для создания макета](#пример-запросов--для-создания-макета)

<!-- /TOC -->

Возможно создание данных на базе специальных табличных документов-макетов.
Использовать можно макеты из внешней обработки-теста или общие макеты или просто табличные документы.

Эти макеты можно генерировать из реальных боевых данных или ввести вручную, в табличный документ.

## Режимы генерации данных при создании объектов и поиске реквизитов объектов

### Режимы создания

Пример обычного макета для создания данных
![image](https://cloud.githubusercontent.com/assets/2920817/5992720/5f5aab0a-aa46-11e4-84df-a0e4dac8d96f.png)

### Режимы поиска

Режим `Выгружать ссылку` - для ссылочных данных выгружает реквизит `Ссылка` как Guid

* Пример макета в режиме `Выгружать ссылку`

![image](https://cloud.githubusercontent.com/assets/2920817/5992725/792740e8-aa46-11e4-9e6c-ca7955c01963.png)

Режим `Связь по Гуид` - для реквизитов выполняет поиск исходного объекта по Guid

* Пример макета в режиме `Выгружать ссылку` и `Связь по Гуид`

![image](https://cloud.githubusercontent.com/assets/2920817/5992741/e7d95684-aa46-11e4-8978-c5b038505a69.png)

## Генерация макета на базе реальных данных

Есть возможность создания табличных макетов из реальных боевых данных.

Для вызова формы создания нужно использовать меню `Внешние инструменты` и команду `Генератор макетов данных` из этого меню. Фактически вызывается внешняя обработка `"Plugins\СериализаторMXL.epf"`

Поддерживается как обычная форма, так и управляемая.

Обычная форма:
![image](https://cloud.githubusercontent.com/assets/2920817/5992794/16b51108-aa49-11e4-9892-9096e5403987.png)

Управляемая форма:
![image](https://cloud.githubusercontent.com/assets/2920817/5992802/8ef7fb6c-aa49-11e4-962b-2bafa1f4ac59.png)

Пример создания макета, который затем можно использовать для создания данных как в пустой/чистовой базе, так и в любой заполненной (тестовая, разработческая, копия рабочей) базе.
![image](https://cloud.githubusercontent.com/assets/2920817/6060933/f50248a6-ad53-11e4-9c90-97e53a1717a2.png)

[Пример кода тестов генерации данных (из репозитария)](/tests/xunit/Plugins/Тесты_СериализаторMXL/Тесты_СериализаторMXL/Ext/ObjectModule.bsl)

### Пример кода создания данных из макета

```bsl
Перем КонтекстЯдра;
Перем СериализаторMXL;

Процедура Инициализация(КонтекстЯдраПараметр) Экспорт
	КонтекстЯдра = КонтекстЯдраПараметр;
	СериализаторMXL = КонтекстЯдра.Плагин("СериализаторMXL");
КонецПроцедуры

Функция ПолучитьСписокТестов() Экспорт
	ВсеТесты = Новый Массив;
	ВсеТесты.Добавить("ТестДолжен_СоздатьПростойСправочник");
	Возврат ВсеТесты;
КонецФункции

Процедура ТестДолжен_СоздатьПростойСправочник() Экспорт
	Макет = ПолучитьМакет("СозданиеПростогоСправочника"); //использую макет из этой же внешней обработки
	ТестовыеДанные = СериализаторMXL.СоздатьДанныеПоТабличномуДокументу(Макет);
	
	ПроверитьСозданиеПростогоСправочника(ТестовыеДанные);
КонецПроцедуры
```

## Создание макета данных на основании результатов запросов (например, через ирПортативные)

Программная генерация макета данных на базе консоли запросов из "Инструменты Разработчика" или собственных запросов из своего кода

### Пример вызова

```bsl
Путь_xUnitFor1C = "C:\Projects\GitHub\xUnitFor1C\";
ПутьГенератораМакета = Путь_xUnitFor1C + "ГенерацияМакетаДанных_На_БазеРеальныхДанных.epf";

СоздательМакета = ВнешниеОбработки.Создать(ПутьГенератораМакета );
Макет = СоздательМакета.СоздатьМакетДанныхНаОснованииЗапроса(РезультатЗапроса);
Макет.Показать();
```

Открываем консоль запросов ИР (или ирПортативные), создаем запрос, Выполняем его
Далее закладка "Обработка результата" (не путать со "Обработка строки результата"), вставляем текст выше и нажимаем кнопку "Выполнить" над кодом (не путать с верхней кнопкой "Выполнить")

## Пример запросов  для создания макета

На базе ссылочных данных (поле __Метаданные должно быть пустым):

```bsl
	Запрос.Текст = 
		"ВЫБРАТЬ
		|	"""" КАК __Метаданные,
		|	""ПростойСправочник"" КАК __Префикс,
		|	ПростойСправочник.Ссылка,
		|	ПростойСправочник.Код,
		|	ПростойСправочник.Наименование,
		|	ПростойСправочник.РеквизитБулево,
		|	ПростойСправочник.РеквизитПеречисление
		|ИЗ
		|	Справочник.ПростойСправочник КАК ПростойСправочник
		|Где РеквизитБулево И ПростойСправочник.Наименование = ""Тестовое наименование"" ";
```

На базе регистров:

```bsl
	Запрос.Текст = 
		"ВЫБРАТЬ
		|	""РегистрыСведений.РегистрСведений1.СоздатьНаборЗаписей().Метаданные()"" КАК __Метаданные,
		|	""ЗаписьРегистра"" КАК __Префикс,
		|	РегистрСведений1.ПростойСправочник,
		|	РегистрСведений1.ПростойСправочник2 КАК ПростойСправочник2,
		|	РегистрСведений1.РесурсЧисло,
		|	РегистрСведений1.РесурсБулево
		|ИЗ
		|	РегистрСведений.РегистрСведений1 КАК РегистрСведений1
		|
		|УПОРЯДОЧИТЬ ПО
		|	ПростойСправочник2";
```
