## Файлы настройки/профиля запуска обработки

Для запуска в консольном режиме используется понятие *профиль консольного запуска*. Профиль консольного запуска предназначен для удобной передачи параметров. Профиль запуска представляет собой текстовый файл в формате JSON.

Профиль запуска предназначен для простого консольного запуска
Пример подобной командной строки выглядит так:

+ `vrunner vanessa --settings tools\JSON\vrunner.json` - BDD
+ `vrunner xunit --settings tools\JSON\vrunner.json` - TDD

### Самый простой и полезный путь запуска

- `vrunner vanessa --settings path_to_your_vrunner.json`

- для этого нужно настроить 3 собственных файла
  - path_to_your_vrunner.json - файл настройки запуска vanessa-runner
  - bdd.json - файл настройки запуска BDD
  - smoke.json или xunit.json - файл настройки запуска TDD или дымовых тестов

Примеры этих файлов смотрите в поставке Vanessa-ADD
  - пример [настройки запуска vanessa-runner tools\JSON\vrunner.json](tools\JSON\vrunner.json)
  - пример [настройки запуска BDD tools\JSON\VBParams8310UF.json](tools\JSON\VBParams8310UF.json)
  - пример [настройки запуска TDD или дымовых тестов tools\JSON\xUnitParams.json](tools\JSON\xUnitParams.json)

Любой из ключей из файлов можно передать в командной строке или через переменные окружения.

Например, запуск дымовых тестов из поставки Ванесса-АДД с указанием специального файла настройки дымовых тестов
- `vrunner xunit $addRoot/tests/smoke --settings tools\vrunner.json --xddConfig ./tools/JSON/xUnitParams.json`

Расширенные примеры запуска можно увидеть в соседнем репозитории [Vanessa-Runner](https://github.com/vanessa-opensource/vanessa-runner)

### Запуск проверки поведения через BDD

или для BDD-запуска
```cmd
%V83PATH% /Execute C:\add\bddRunner.epf /C"StartFeaturePlayer;VBParams=C:\VBParams.json"
```

Текущие параметры настройки для json-файлов из BDD:

+ **Каталог фич** - каталог, где собраны требования заказчика описанные на языке Gherkin
+ **ВыполнитьСценарии** - признак того, что необходимо запустить выполнение сценариев
+ **ДелатьОтчетВФорматеАллюр** - признак того, что необходимо формировать HTML отчёт о результатах проверки
+ **КаталогOutputAllureБазовый** - адрес каталога, для где будет формироваться HTML отчёт
+ **ЗавершитьРаботуСистемы** - признак того, что окончанию работы необходимо завершить работу 1С предприятия
+ **ВыгружатьСтатусВыполненияСценариевВФайл** - признак, что необходимо формировать файл с финальным статусом проверки
+ **ПутьКФайлуДляВыгрузкиСтатуасВыполненияСценариев** - по данному пути будет сформирован файл со статусом проверки (обычно используется на серверах сборки для автоматизированного указания статуса сборки)
+ **СписокТеговИсключение** - массив текстовых тэгов для исключения из проверки (например, используется для черновиков сценариев и требований)
+ **СписокТеговОтбор** - массив текстовых тэгов для запуска проверки поведения по сценариям, содержащим любой из указанных тэгов

Пример подобного JSON файла профиля:

```json
{
"КаталогФич": "C:\\add\features",
"ВыполнитьСценарии": "Истина",
"ДелатьОтчетВФорматеАллюр": "Истина",
"КаталогOutputAllureБазовый": "C:\\allurereport",
"ЗавершитьРаботуСистемы": "Истина",
"ВыгружатьСтатусВыполненияСценариевВФайл": "Истина",
"ПутьКФайлуДляВыгрузкиСтатусаВыполненияСценариев": "C:\\BuildStatus.log",
"СписокТеговИсключение":[
  "IgnoreOnCIMainBuild",
  "Draft"
]
}
```

Много примеров json-файлов есть в каталоге `tools` проекта.
